import { Component, createEffect, onCleanup } from 'solid-js';

import VectorSource from 'ol/source/Vector';
import GeoJSON from 'ol/format/GeoJSON';
import { useI18n } from '@solid-primitives/i18n';
import { cloneDeep } from 'lodash';
import {
  createCachedSignal,
  destroyCachedSignal,
} from '../../workers/cached-signals';

interface Props {
  wptId: string;
  vectorSource: VectorSource;
  context: any;
}

export const Wpt: Component<Props> = ({
  vectorSource,
  wptId: wptId,
  context,
}) => {
  const [t, { add, locale, dict }] = useI18n();

  const params = {
    id: wptId,
    method: 'getWpt',
  };
  const wpt = createCachedSignal(params);

  onCleanup(() => {
    console.log({
      caller: 'Wpt / onCleanup',
      wptId,
      wpt: wpt(),
      params,
    });

    destroyCachedSignal(params);
  });

  const getTitle = () => {
    if (wpt()?.name) {
      return `${wpt()?.name} (${t('wpt')})`;
    }
    return t('wpt');
  };

  createEffect(() => {
    console.log({ caller: 'Wpt', vectorSource, wptId, wpt: cloneDeep(wpt()) });

    if (wpt()) {
      const existingFeature = vectorSource.getFeatureById(wptId);
      if (existingFeature) {
        vectorSource.removeFeature(existingFeature);
      }
      let geo: any = {
        type: 'FeatureCollection',
        features: [
          {
            type: 'Feature',
            geometry: {
              type: 'Point',
              coordinates: [wpt()?.$.lon, wpt()?.$.lat],
            },
            properties: {
              type: 'wpt',
              ...wpt(),
              id: wptId,
              getTitle,
              context: { ...context, wpt, wptId },
            },
            id: wptId,
          },
        ],
      };

      const features = new GeoJSON().readFeatures(geo);
      console.log({ caller: 'Wpt', features });
      const feature = vectorSource.getFeatureById(wptId);
      if (feature) {
        vectorSource.removeFeature(feature);
      }
      vectorSource.addFeatures(features);
    }
  });

  return <></>;
};

export default Wpt;
